import json
from logs.logger import *
from service.standard_service import StandardService
from data_initialize_standard.results import *
from communication.rabbitmq_link import create_connect, send_data_to_rabbitmq
# from communication.rocketmq_link import start_rocket, send_data_to_rocket


def main(data_input):
    dict_input = json.loads(data_input)
    logger.critical("============Get data from rabbitmq============")
    logger.critical("Input: {}".format(json.dumps(dict_input, ensure_ascii=False)))
    try:
        if dict_input['controlConfigInfo']['handleService'] == 1:
            logger.critical('============算法服务开启，进入水温智控算法============')
            mcss = StandardService(dict_input)
            dict_results = mcss.main_multi_comp_service()
        else:
            logger.critical('============算法服务关闭，退出水温智控算法============')
            dict_results = {}
        logger.critical("============Send calculated results to rabbitmq============")
        dict_results = json.dumps(dict_results, ensure_ascii=False)
        logger.critical("Output: {}".format(dict_results))
        send_data_to_rabbitmq(dict_results)

    except Exception as e:
        result_depict = '异常中断，退出水温智控算法' + "(" + str(e) + ")"
        logger.critical('============{}============'.format(result_depict))
        dict_results = DictResultsAbnormal(dict_input, result_depict)
        logger.critical("============Send calculated results to rabbitmq============")
        dict_results = json.dumps(dict_results, ensure_ascii=False, default=default_dump)
        logger.critical("Output: {}".format(dict_results))
        send_data_to_rabbitmq(dict_results)
        # send_data_to_rocket(dict_results)


if __name__ == '__main__':
    logger.critical("========Create connection for rabbitmq========")
    create_connect()
    # start_rocket()


