# from logs_conf.logger import *
from logs.logger import *
import math


class ModifyLeavingWaterTempSetTSS10(object):
    def __init__(self, water_temp_set_new, dict_chiller_inner):
        self.water_temp_set_new = water_temp_set_new
        self.dict_chiller_inner = dict_chiller_inner

    def modify_temp(self):
        if self.dict_chiller_inner['runMode'] == 0:
            result = self.modify_temp_cooling()
        elif self.dict_chiller_inner['runMode'] == 1:
            result = self.modify_temp_heating()
        else:
            result = ''
            logger.critical('============冷机运行模式输入错误============')
        return result

    def modify_temp_cooling(self):
        if self.water_temp_set_new > self.dict_chiller_inner['chillerWaterTempSet']:
            result = self.increase_temp_cooling()
        else:
            result = self.decrease_temp_cooling()
        return result

    def modify_temp_heating(self):
        if self.water_temp_set_new < self.dict_chiller_inner['coolingWaterTempOut']:
            result = self.decrease_temp_heating()
        else:
            result = self.increase_temp_heating()
        return result

    def increase_temp_cooling(self):
        if math.ceil(self.water_temp_set_new) - self.dict_chiller_inner['chillerWaterTempOut'][-1] > \
                0.75 * self.dict_chiller_inner['coolingShutdownOffset']:
            deltaWaterTempSet = round(math.floor(self.water_temp_set_new) - self.dict_chiller_inner['chillerWaterTempSet'], 2)
            if deltaWaterTempSet >= 1:
                result = self.dict_chiller_inner['chillerWaterTempSet'] + 1
            else:
                result = self.dict_chiller_inner['chillerWaterTempSet']
        else:
            deltaWaterTempSet = round(self.water_temp_set_new - self.dict_chiller_inner['chillerWaterTempSet'], 2)
            if deltaWaterTempSet >= 1:
                result = self.dict_chiller_inner['chillerWaterTempSet'] + 1
            elif self.water_temp_set_new < 10 and deltaWaterTempSet > 0.1:
                result = math.ceil(self.water_temp_set_new)
            else:
                if deltaWaterTempSet >= 0.5:
                    result = math.ceil(self.water_temp_set_new)
                else:
                    result = math.floor(self.water_temp_set_new)
        return result

    def decrease_temp_cooling(self):
        deltaWaterTempSet = round(self.water_temp_set_new - self.dict_chiller_inner['chillerWaterTempSet'], 2)
        if abs(deltaWaterTempSet) >= 1:
            fractionalPart = abs(deltaWaterTempSet) % 1
            if fractionalPart >= 0.5:
                result = self.dict_chiller_inner['chillerWaterTempSet'] - math.ceil(abs(deltaWaterTempSet))
            else:
                result = self.dict_chiller_inner['chillerWaterTempSet'] - math.floor(abs(deltaWaterTempSet))
        elif abs(deltaWaterTempSet) >= 0.25:
            result = math.floor(self.water_temp_set_new)
        else:
            result = math.ceil(self.water_temp_set_new)  # 制冷模式的降温过程向上取整即取原值
        return result

    def increase_temp_heating(self):
        deltaWaterTempSet = round(self.dict_chiller_inner['coolingWaterTempOut'] - self.water_temp_set_new, 2)
        if abs(deltaWaterTempSet) >= 1:
            fractionalPart = abs(deltaWaterTempSet) % 1
            if fractionalPart >= 0.5:
                result = self.dict_chiller_inner['coolingWaterTempOut'] + math.ceil(abs(deltaWaterTempSet))
            else:
                result = self.dict_chiller_inner['coolingWaterTempOut'] + math.floor(abs(deltaWaterTempSet))
        elif abs(deltaWaterTempSet) >= 0.5:
            result = math.ceil(self.water_temp_set_new)
        else:
            result = math.floor(self.water_temp_set_new)  # 制热模式的升温过程向下取整即取原值
        return result

    def decrease_temp_heating(self):
        if self.dict_chiller_inner['coolingWaterTempOut'] - math.floor(self.water_temp_set_new) > \
                0.75 * self.dict_chiller_inner['heatingShutdownOffset']:
            deltaWaterTempSet = round(self.dict_chiller_inner['coolingWaterTempOut'] - math.ceil(self.water_temp_set_new), 2)
            if deltaWaterTempSet >= 1:
                result = self.dict_chiller_inner['coolingWaterTempOut'] - 1
            else:
                result = self.dict_chiller_inner['coolingWaterTempOut']
        else:
            deltaWaterTempSet = round(self.dict_chiller_inner['coolingWaterTempOut'] - self.water_temp_set_new, 2)
            if deltaWaterTempSet >= 1:
                result = self.dict_chiller_inner['coolingWaterTempOut'] - 1
            elif self.water_temp_set_new > 50 and deltaWaterTempSet > 0.1:
                result = math.floor(self.water_temp_set_new)
            else:
                if deltaWaterTempSet >= 0.5:
                    result = math.floor(self.water_temp_set_new)
                else:
                    result = math.ceil(self.water_temp_set_new)
        return result


class ModifyReturningWaterTempSetTSS10(object):
    def __init__(self, water_temp_set_new, dict_chiller_inner):
        self.water_temp_set_new = water_temp_set_new
        self.dict_chiller_inner = dict_chiller_inner

    def modify_temp(self):
        if self.dict_chiller_inner['runMode'] == 0:
            result = self.modify_temp_cooling()
        elif self.dict_chiller_inner['runMode'] == 1:
            result = ''
            logger.critical('============制热模式和回水温度控制模式下温度修正函数未开发, Error: TSS10============')
        else:
            result = ''
            logger.critical('============冷机运行模式输入错误============')
        return result

    def modify_temp_cooling(self):
        if self.water_temp_set_new > self.dict_chiller_inner['chillerWaterTempInSet']:
            result = self.increase_temp_cooling()
        else:
            result = self.decrease_temp_cooling()
        return result

    def increase_temp_cooling(self):
        if math.ceil(self.water_temp_set_new) - self.dict_chiller_inner['chillerWaterTempIn'][-1] > \
                0.75 * self.dict_chiller_inner['coolingShutdownOffset']:
            deltaWaterTempSet = round(math.floor(self.water_temp_set_new) - self.dict_chiller_inner['chillerWaterTempInSet'], 2)
            if deltaWaterTempSet >= 1:
                result = self.dict_chiller_inner['chillerWaterTempInSet'] + 1
            else:
                result = self.dict_chiller_inner['chillerWaterTempInSet']
        else:
            deltaWaterTempSet = round(self.water_temp_set_new - self.dict_chiller_inner['chillerWaterTempInSet'], 2)
            if deltaWaterTempSet >= 1:
                result = self.dict_chiller_inner['chillerWaterTempInSet'] + 1
            elif self.water_temp_set_new < 10 and deltaWaterTempSet > 0.1:
                result = math.ceil(self.water_temp_set_new)
            else:
                if deltaWaterTempSet >= 0.5:
                    result = math.ceil(self.water_temp_set_new)
                else:
                    result = math.floor(self.water_temp_set_new)
        return result

    def decrease_temp_cooling(self):
        deltaWaterTempSet = round(self.water_temp_set_new - self.dict_chiller_inner['chillerWaterTempInSet'], 2)
        if abs(deltaWaterTempSet) >= 1:
            fractionalPart = abs(deltaWaterTempSet) % 1
            if fractionalPart >= 0.5:
                result = self.dict_chiller_inner['chillerWaterTempInSet'] - math.ceil(abs(deltaWaterTempSet))
            else:
                result = self.dict_chiller_inner['chillerWaterTempInSet'] - math.floor(abs(deltaWaterTempSet))
        elif abs(deltaWaterTempSet) >= 0.25:
            result = math.floor(self.water_temp_set_new)
        else:
            result = math.ceil(self.water_temp_set_new)
        return result
