from service.public_method import PublicMethodClass
from chiller_safety_modify.multi_comp_safety_verify import MultiCompSafetyVerify
from data_initialize_standard.data_standardization import DataStandardization
from data_initialize_standard.results import *
from logs.logger import *


class StandardService(PublicMethodClass):
    def __init__(self, dict_input):
        super(StandardService, self).__init__(dict_input)
        self.dict_input = dict_input

    def main_multi_comp_service(self):
        # 数据校核
        logger.critical("============开始对输入数据进行完整性校验============")
        self.ds = DataStandardization(self.dict_input)
        missing_fields, empty_paths = self.ds.check_data()

        # 判断输入数据是否满足计算条件，且智控开关开启
        if missing_fields or empty_paths or self.dict_input['controlConfigInfo']['controlMode'] != 1:
            self.output_abnormal_situation(missing_fields, empty_paths)
        else:
            logger.critical("============输入数据完整性校验通过============")
            # 数据校核正常，开启计算
            self.dict_chiller_inner, self.dict_chiller_outer, self.dict_code, self.dict_upper_correct, \
            self.data_temp_humi = self.ds.data_multi_comp()
            # 运行安全校验
            logger.critical("============开始设备安全运行校验============")
            mcsv = MultiCompSafetyVerify(self.dict_chiller_inner, self.dict_upper_correct)
            is_safety_issue, self.water_temp_set_new = mcsv.safety_verify_main_process()
            # 控制周期和控制条件判断
            logger.critical("============开始控制周期和控制条件判断============")
            is_control = self.control_period_cal(is_safety_issue)
            if is_control == 1:
                # 满足控制条件
                if self.water_temp_set_new:
                    # 存在安全问题
                    self.water_temp_set_new = round(self.water_temp_set_new, 1)
                    # 水温设定值校验
                    logger.critical("============开始水温设定值校核============")
                    self.water_temp_set_new = self.water_temp_set_verify(True)
                    self.isAdjustWaterTemp = 1
                else:
                    # 不存在安全问题，根据监测点进行水温计算
                    self.water_temp_set_new = self.water_temp_main_process()
                    if self.water_temp_set_new:
                        # 水温设定值校验
                        logger.critical("============开始水温设定值校核============")
                        self.water_temp_set_new = round(self.water_temp_set_new, 1)
                        self.water_temp_set_verify(self.terminal_cal)
                        # 判断是否要调节水温
                        logger.critical("============开始水温调整条件判断============")
                        self.is_adjust_judge()
                self.output_normal_situation(is_safety_issue)
            else:
                self.output_abnormal_situation(missing_fields, empty_paths, is_control)
        return self.dict_results
